*Safety Layout Problem with 9 rectangles
*Big-M version

$TITLE Safety Layout Problem
$OFFSYMXREF
$OFFSYMLIST
$ONINLINE

SETS
I /1*9/ /*Number of rectangles*/
D1 /1*4/ /*Number of disjuncts for non-overlapping constraints*/
;

ALIAS
(I,J)
;

PARAMETERS
L(I)
        / 1 = 5
          2 = 7
          3 = 3
          4 = 2
          5 = 4
          6 = 5
          7 = 8
          8 = 4
          9 = 2/

H(I)
        / 1 = 6
          2 = 5
          3 = 3
          4 = 3
          5 = 4
          6 = 2
          7 = 6
          8 = 6
          9 = 5/

PARAMETER COST(I)
         / 1    150
           2    390
           3    240
           4     70
           5    165
           6    100
           7    200
           8    400
           9    330/;

PARAMETER XO(I)
         / 1     4
           2     10
           3     7
           4     3
           5     20
           6     18
           7     30
           8     24
           9     22/;

PARAMETER YO(I)
         / 1     10
           2     15
           3     9
           4     3
           5     17
           6     8
           7     20
           8     10
           9     6/;

SCALARS
LENGTH /40/
WIDTH /40/;

PARAMETERS
XMIN(I) /1*9 = 0/
XMAX(I) /1*9 = 0/
YMIN(I) /1*9 = 0/
YMAX(I) /1*9 = 0/;

XMIN(I) = L(I)/2;
XMAX(I) = LENGTH - L(I)/2;
YMIN(I) = H(I)/2;
YMAX(I) = WIDTH - H(I)/2;

PARAMETER
M1(D1,I,J) /1*4 .1*9 .1*9 = 0/;

M1('1',I,J)$(ORD(I) LT ORD(J)) = XMAX(I) + L(I)/2 - XMIN(J) + L(J)/2;
M1('2',I,J)$(ORD(I) LT ORD(J)) = XMAX(J) + L(J)/2 - XMIN(I) + L(I)/2;
M1('3',I,J)$(ORD(I) LT ORD(J)) = YMAX(I) + H(I)/2 - YMIN(J) + H(J)/2;
M1('4',I,J)$(ORD(I) LT ORD(J)) = YMAX(J) + H(J)/2 - YMIN(I) + H(I)/2;

TABLE C(I,J)

             1       2       3       4       5       6        7        8        9
      1              300     240     210     140     300      250      300      210
      2                      100     150     220     200      300      290      400
      3                              120     300     150      150      100      290
      4                                      100     120      180      220      110
      5                                              130      190      110      160
      6                                                       220      140      120
      7                                                                260      220
      8                                                                         140
;

VARIABLES
X(I)
Y(I)
Z(D1,I,J)
DELX(I,J)
DELY(I,J)
OBJ;

POSITIVE VARIABLES
X(I)
Y(I)
DELX(I,J)
DELY(I,J)
;

BINARY VARIABLES
Z(D1,I,J);

EQUATIONS
OBJBM

DIST1(I,J)
DIST2(I,J)
DIST3(I,J)
DIST4(I,J)

DISJ1(I,J)
DISJ2(I,J)
DISJ3(I,J)
DISJ4(I,J)
BIN1(I,J)
;

*Objective Function
OBJBM.. OBJ =E= SUM((I,J),C(I,J)*(DELX(I,J) + DELY(I,J))) + SUM(I,COST(I)*(SQR(X(I)-XO(I))+SQR(Y(I)-YO(I))));

*Distance Constraints
DIST1(I,J)$(ORD(I) LT ORD(J)).. DELX(I,J) =G= X(I) - X(J);
DIST2(I,J)$(ORD(I) LT ORD(J)).. DELX(I,J) =G= X(J) - X(I);
DIST3(I,J)$(ORD(I) LT ORD(J)).. DELY(I,J) =G= Y(I) - Y(J);
DIST4(I,J)$(ORD(I) LT ORD(J)).. DELY(I,J) =G= Y(J) - Y(I);

*Disjunctions - Overlap
DISJ1(I,J)$(ORD(I) LT ORD(J)).. X(I) + L(I)/2  =L= X(J) - L(J)/2 + M1('1',I,J)*(1-Z('1',I,J));
DISJ2(I,J)$(ORD(I) LT ORD(J)).. X(J) + L(J)/2  =L= X(I) - L(I)/2 + M1('2',I,J)*(1-Z('2',I,J));
DISJ3(I,J)$(ORD(I) LT ORD(J)).. Y(I) + H(I)/2  =L= Y(J) - H(J)/2 + M1('3',I,J)*(1-Z('3',I,J));
DISJ4(I,J)$(ORD(I) LT ORD(J)).. Y(J) + H(J)/2  =L= Y(I) - H(I)/2 + M1('4',I,J)*(1-Z('4',I,J));

*Summation of binary variables
BIN1(I,J)$(ORD(I) LT ORD(J)).. SUM(D1,Z(D1,I,J)) =E= 1;

*Bound Constraints
X.LO(I) = XMIN(I);
Y.LO(I) = YMIN(I);
X.UP(I) = XMAX(I);
Y.UP(I) = YMAX(I);

*Model parameters
MODEL LAYOUT_BM / ALL /;

OPTION MINLP = sbb;
OPTION RMINLP = CONOPT;
OPTION MIP = CPLEX;
OPTION ITERLIM = 100000000;
OPTION RESLIM = 10800;
OPTION OPTCR    = 0.0 ;
OPTION OPTCA    = 0.0 ;
OPTION LIMROW   = 0 ;
OPTION LIMCOL   = 0 ;
*OPTION SOLPRINT = OFF ;
*OPTION SYSOUT   = OFF ;

*SOLVE LAYOUT_BM USING RMINLP MINIMIZING OBJ;

LAYOUT_BM.OPTFILE = 1;
LAYOUT_BM.NODLIM = 1000000;
SOLVE LAYOUT_BM USING MINLP MINIMIZING OBJ;