*Safety Layout Problem with 10 rectangles
*Convex Hull version

$TITLE Safety Layout Problem
$OFFSYMXREF
$OFFSYMLIST
$ONINLINE

SETS
I /1*10/ /*Number of rectangles*/
D1 /1*4/ /*Number of disjuncts for non-overlapping constraints*/
;

ALIAS
(I,J,K)
;

PARAMETERS
L(I)
        / 1 = 5
          2 = 7
          3 = 3
          4 = 2
          5 = 4
          6 = 5
          7 = 8
          8 = 4
          9 = 2
         10 = 6/

H(I)
        / 1 = 6
          2 = 5
          3 = 3
          4 = 3
          5 = 4
          6 = 2
          7 = 6
          8 = 6
          9 = 5
         10 = 3/

PARAMETER COST(I)
         / 1    150
           2    390
           3    240
           4     70
           5    165
           6    100
           7    200
           8    400
           9    330
          10    220/;

PARAMETER XO(I)
         / 1     4
           2     10
           3     7
           4     3
           5     20
           6     18
           7     30
           8     24
           9     22
          10     11/;

PARAMETER YO(I)
         / 1     10
           2     15
           3     9
           4     3
           5     17
           6     8
           7     20
           8     10
           9     6
          10     12/;

SCALARS
LENGTH /40/
WIDTH /40/;

PARAMETERS
XMIN(I) /1*10 = 0/
XMAX(I) /1*10 = 0/
YMIN(I) /1*10 = 0/
YMAX(I) /1*10 = 0/;

XMIN(I) = L(I)/2;
XMAX(I) = LENGTH - L(I)/2;
YMIN(I) = H(I)/2;
YMAX(I) = WIDTH - H(I)/2;

TABLE C(I,J)

             1       2       3       4       5       6        7        8        9        10
      1              300     240     210     140     300      250      300      210      320
      2                      100     150     220     200      300      290      400      220
      3                              120     300     150      150      100      290      110
      4                                      100     120      180      220      110      100
      5                                              130      190      110      160      400
      6                                                       220      140      120      230
      7                                                                260      220      310
      8                                                                         140      150
      9                                                                                  130
;

SCALAR  ES  /1E-6/;

VARIABLES
X(I)
Y(I)
V(K,D1,I,J)
VV(K,D1,I,J)
Z(D1,I,J)
DELX(I,J)
DELY(I,J)
OBJ;

POSITIVE VARIABLES
X(I)
Y(I)
V(K,D1,I,J)
VV(K,D1,I,J)
DELX(I,J)
DELY(I,J)
;

BINARY VARIABLES
Z(D1,I,J);

EQUATIONS
OBJCH
DIST1(I,J)
DIST2(I,J)
DIST3(I,J)
DIST4(I,J)
DISAGGX_DISJ1(K,I,J)
DISAGGY_DISJ1(K,I,J)
BNDV(K,D1,I,J)
BNDVV(K,D1,I,J)
DISJ1(I,J)
DISJ2(I,J)
DISJ3(I,J)
DISJ4(I,J)
BIN1(I,J);

*Objective Function
OBJCH.. OBJ =E= SUM((I,J),C(I,J)*(DELX(I,J) + DELY(I,J))) + SUM(I,COST(I)*(SQR(X(I)-XO(I))+SQR(Y(I)-YO(I))));

*Distance Constraints
DIST1(I,J)$(ORD(I) LT ORD(J)).. DELX(I,J) =G= X(I) - X(J);
DIST2(I,J)$(ORD(I) LT ORD(J)).. DELX(I,J) =G= X(J) - X(I);
DIST3(I,J)$(ORD(I) LT ORD(J)).. DELY(I,J) =G= Y(I) - Y(J);
DIST4(I,J)$(ORD(I) LT ORD(J)).. DELY(I,J) =G= Y(J) - Y(I);

*Disaggregated variables
DISAGGX_DISJ1(K,I,J)$(ORD(I) LT ORD(J) AND (ORD(K) EQ ORD(I) OR ORD(K) EQ ORD(J))).. X(K) =E= SUM(D1,V(K,D1,I,J));
DISAGGY_DISJ1(K,I,J)$(ORD(I) LT ORD(J) AND (ORD(K) EQ ORD(I) OR ORD(K) EQ ORD(J))).. Y(K) =E= SUM(D1,VV(K,D1,I,J));

*Bounds on disaggregated variables
BNDV(K,D1,I,J)$(ORD(I) LT ORD(J) AND (ORD(K) EQ ORD(I) OR ORD(K) EQ ORD(J))).. V(K,D1,I,J) =L= Z(D1,I,J)*XMAX(I);
BNDVV(K,D1,I,J)$(ORD(I) LT ORD(J) AND (ORD(K) EQ ORD(I) OR ORD(K) EQ ORD(J))).. VV(K,D1,I,J) =L= Z(D1,I,J)*YMAX(I);

*Disjunctions - Overlap
DISJ1(I,J)$(ORD(I) LT ORD(J)).. V(I,'1',I,J) - V(J,'1',I,J) =L= (-L(I)/2 - L(J)/2)*Z('1',I,J);
DISJ2(I,J)$(ORD(I) LT ORD(J)).. V(J,'2',I,J) - V(I,'2',I,J) =L= (-L(J)/2 - L(I)/2)*Z('2',I,J);
DISJ3(I,J)$(ORD(I) LT ORD(J)).. VV(I,'3',I,J) - VV(J,'3',I,J) =L= (-H(I)/2 - H(J)/2)*Z('3',I,J);
DISJ4(I,J)$(ORD(I) LT ORD(J)).. VV(J,'4',I,J) - VV(I,'4',I,J) =L= (-H(J)/2 - H(I)/2)*Z('4',I,J);

*Summation of binary variables
BIN1(I,J)$(ORD(I) LT ORD(J)).. SUM(D1,Z(D1,I,J)) =E= 1;

*Bound Constraints
X.LO(I) = XMIN(I);
Y.LO(I) = YMIN(I);
X.UP(I) = XMAX(I);
Y.UP(I) = YMAX(I);

*Model parameters
MODEL LAYOUT_CH / ALL /;

OPTION MINLP = sbb;
OPTION RMINLP = CONOPT;
OPTION MIP = CPLEX;
OPTION ITERLIM = 100000000;
OPTION RESLIM = 10800;
OPTION OPTCR    = 0.0 ;
OPTION OPTCA    = 0.0 ;
OPTION LIMROW   = 0 ;
OPTION LIMCOL   = 0 ;
*OPTION SOLPRINT = OFF ;
*OPTION SYSOUT   = OFF ;

*LAYOUT_CH.OPTFILE = 1;
*SOLVE LAYOUT_CH USING RMINLP MINIMIZING OBJ;

LAYOUT_CH.OPTFILE = 1;
LAYOUT_CH.NODLIM = 1000000;
SOLVE LAYOUT_CH USING MINLP MINIMIZING OBJ;